package de.hsbo.fbv.bmg.geometry.simple2hd.buch;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Shape;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JFrame;

import de.hsbo.fbv.bmg.geometry.simple.GPoint;
import de.hsbo.fbv.bmg.geometry.simple.GTriangle;
import de.hsbo.fbv.bmg.geometry.simple.collections.GPointSet;
import de.hsbo.fbv.bmg.geometry.simple2hd.demos.DataGenerator;
import de.hsbo.fbv.bmg.geometry.simple2hd.demos.DelaunayTest;
import de.hsbo.fbv.bmg.graphics.viewer.simple.Map;
import de.hsbo.fbv.bmg.graphics.viewer.simple.ShapeLayer;

/**
 * Dies Klasse demonstriert die Genererierung eines Dreiecksnetzes aus einer Menge 
 * gegebener Punkte.
 * @author A.Zimmermann, HS Bochum, 2012
 *
 */
public class DelaunayFrame extends JFrame {
	
	public static void main(String[] args) {
		DelaunayFrame df = new DelaunayFrame();
		df.setDefaultCloseOperation(EXIT_ON_CLOSE);
		df.setVisible(true);
	}
	
	// Attribute
	Map map = new Map();
	ShapeLayer<GPoint> ptl;
	ShapeLayer<GTriangle> tril;
	ShapeLayer<Shape> circles;
	ShapeLayer<Shape> last; 
	// GPoint[] pta = DataGenerator.generatePoints(50, 400, 400, 200, 200, 1);
	GPoint[] pta = DataGenerator.generateGridPoints(500, 500, 200, 200, 40, 5);
	List<GTriangle> tin;
	int nstep = 1;

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;

	/**
	 * This is the default constructor
	 */
	public DelaunayFrame() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(800, 600);
		this.setContentPane(getJContentPane());
		this.setTitle("Delaunay Demo - WEITER: Linke Maustaste");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = map;
			ptl = getPointLayer();
			tril = getTriangleLayer();
			circles = getCircleLayer();
			map.add(tril);
			map.add(ptl);
			map.add(circles);
			/*
			if(last!=null)
				map.add(last); */
			jContentPane.setLayout(new BorderLayout());
			jContentPane.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(java.awt.event.MouseEvent e) {
					nstep++;
					map.remove(tril);
					map.remove(circles);
					map.remove(last);
					
					tril = getTriangleLayer();
					map.add(tril);
					map.add(ptl);
					circles = getCircleLayer();
					map.add(circles);
					last = getLastLayer();
					map.add(last);
					repaint();
				}
			});
		}
		return jContentPane;
	}
	
	private ShapeLayer<GPoint> getPointLayer() {
		ShapeLayer<GPoint> l = new ShapeLayer<GPoint>();
		for (int i = 0; i < pta.length; i++) {
			l.add(pta [i]);
		}
		l.setDrawColor(Color.black);
		l.setStroke(new BasicStroke(2.f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		return l;
	}
	
	private ShapeLayer<GTriangle> getTriangleLayer() {
		ShapeLayer<GTriangle> l = new ShapeLayer<GTriangle>();
		GPointSet pset = new GPointSet(pta);
		tin = DelaunayTest.getTIN(pset, nstep);
		for (int i = 0; i < tin.size(); i++) {
			l.add(tin.get(i));
		}
		l.setDrawColor(new Color(0,0xc0,0));
		l.setStroke(new BasicStroke(0.2f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		return l;
	}
	
	private ShapeLayer<Shape> getCircleLayer() {
		ShapeLayer<Shape> l = new ShapeLayer<Shape>();
		GTriangle lastT = DelaunayTest.lastT;
		GTriangle[] next = DelaunayTest.lastT.getNextTriangles();
		for(int i=0; i<next.length; i++)
			if(next[i]!=null) {
				l.add(((GTriangle)next[i]).getCircleShape());
				l.add(next[i]);
			}
		l.setDrawColor(Color.red);
		l.setStroke(new BasicStroke(0.2f));
		return l;
	}

	private ShapeLayer<Shape> getLastLayer() {
		
		if (DelaunayTest.lastT!=null && DelaunayTest.lastP!=null) {
			ShapeLayer<Shape> l = new ShapeLayer<Shape>();
			l.add(DelaunayTest.lastT);
			l.add(DelaunayTest.lastP);
			l.setDrawColor(Color.red);
			l.setStroke(new BasicStroke(1.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
			return l;
		}
		else 
			return null;
	}

}
