package de.hsbo.fbv.bmg.geometry.simple2hd.buch;

import de.hsbo.fbv.bmg.geometry.simple.GLinearRing;
import de.hsbo.fbv.bmg.geometry.simple.GPoint;
import de.hsbo.fbv.bmg.geometry.simple.GTriangle;
import de.hsbo.fbv.bmg.geometry.simple.GeoModel;
import Jama.Matrix;

/**
 * Klasse, die statische Methoden zur Interpolation von Oberflchen bereitstsellt.
 * @author A.Zimmermann, HS Bochum, 2011
 *
 */
public class Interporlation2hD {
	
	/**
	 * Berechnet die interpolierten Z-Werte fr alle Punkte, die innerhalb des Dreiecks triangle
	 * liegen und speichert sie in einem neuen Punktfeld. <br>
	 * Achtung: durch Refactoring der Klassenblibliotheken wurde der Quellcode gegenber
	 * dem Listing 4.9 abgendert.
	 * @param triangle - gegebenes Dreieck
	 * @param points - Feld der Punkte, die interpoliert werden sollen
	 * @return - Feld der neuen, interpolierten Punkte
	 */
	public static GPoint[] getInterpolatedPointsZ(GTriangle triangle, GPoint[] points) 
			throws Exception {
		// Daten vorbereiten
		GPoint[] tPts = ((GLinearRing) triangle.exteriorRing()).getPoints();
		GPoint pa = tPts[0];
		GPoint pb = tPts[1];
		GPoint pc = tPts[2];
		// Prfen, ob die Dreieckpunkte gltige Z-Werte der Gre haben
		for (int i = 0; i < tPts.length; i++) {
			if (Double.isNaN(tPts[i].z()) || Double.isInfinite(tPts[i].z()))
				throw new Exception("Z-value of " + tPts[i] + " is missing");
		}
		// Matrix der Dreieckpunkte aufstellen
		double[][] mx = { { pb.x() - pa.x(), pc.x() - pa.x(), 0 },
				{ pb.y() - pa.y(), pc.y() - pa.y(), 0 },
				{ pb.z() - pa.z(), pc.z() - pa.z(), -1 } };
		// Matrix der Interpolationspunkte aufstellen
		double[][] my = new double[3][points.length];
		for (int i = 0; i < points.length; i++) {
			my[0][i] = points[i].x() - pa.x();
			my[1][i] = points[i].y() - pa.y();
			my[2][i] = -pa.z();
		}
		// Gleichungssystem aufstellen und lsen
		double[][] mz = solveEquations(mx, my);
		// Lsungsvektor auswerten
		GPoint[] results = new GPoint[points.length];
		for (int i = 0; i < points.length; i++) {
			double t1 = mz[0][i];
			double t2 = mz[1][i];
			double z;
			// Wenn Punkt innerhalb des Dreiecks
			if (t1 >= 0 && t2 >= 0 && (t1 + t2) <= 1)
				z = mz[2][i];
			// sonst
			else
				z = Double.NaN;
			// Ursprnglichen Punkt kopieren
			results[i] = GeoModel.createPoint(points[i]);
			// Z-Komponente einfgen
			((GPoint) results[i]).setZ(z);
		}
		return results;
	}

	/**
	 * Lst das Gleichungssystem mit Hilfe der Matrix-Bibliothek jama.jar
	 * @param x
	 * @param y
	 * @param z
	 * @return
	 */
	private static double[][] solveEquations(double[][] x, double[][] y) {
		Matrix X = new Matrix(x);
		Matrix Y = new Matrix(y);
		Matrix Z = X.solve(Y);
		double[][]z = Z.getArray();
		return z;
	}
	
	
}
