package de.hsbo.fbv.bmg.geometry.simple2hd.demos;

import java.util.ArrayList;
import java.util.List;

import de.hsbo.fbv.bmg.geometry.simple.CoordinateM;
import de.hsbo.fbv.bmg.geometry.simple.GPoint;
import de.hsbo.fbv.bmg.geometry.simple.GTriangle;
import de.hsbo.fbv.bmg.geometry.simple.GeoModel;
import de.hsbo.fbv.bmg.geometry.simple.collections.GPointSet;



/**
 * Klasse zum Generieren von Datenbstnden
 * @author zimmermann
 *
 */
public class DataGenerator {
	
	/**
	 * Generiert ein Feld mit n zufallsverteilten Punkten.
	 * @param n			Anzahl der Punkte
	 * @param minx		x-Koordinate der linken unteren Ecke	
	 * @param miny		y-Koordinate der linken unteren Ecke
	 * @param spanx		Ausdehnung in x-Richtung
	 * @param spany		Ausdehnung in y-Richtung
	 * @param res		Auflsung der x- und y-Werte
	 * @return			GPoint[]
	 */
	public static GPoint[] generatePoints(int n, double minx, double miny, 
			double spanx, double spany, double res) {
		
		GPoint[] points = new GPoint[n];
		for(int i=0; i<points.length; i++)
			points[i] = GeoModel.createPoint(
					minx + Math.floor(Math.random()*spanx/res)*res,
					miny + Math.floor(Math.random()*spany/res)*res);
		return points;
	}
	
	
	/**
	 * Erzeugt ein Punktfeld mit rasterfrmiger Anordnung der Punkte.
	 * @param minx		kleinster x-Wert
	 * @param miny		kleinster y-Wert
	 * @param spanx 	Ausdehnung in x
	 * @param spany 	Ausdehnung in y
	 * @param d 		Rasterweite
	 * @param r			Wert der zufallsverteilten Streuung der Punktlage
	 * @return
	 */
	public static GPoint[] generateGridPoints(double minx, double miny, 
			double spanx, double spany, double d, double r) {
		
		GPointSet pset = new GPointSet();
		for(double x = minx; x<minx+spanx; x=x+d) {
			for(double y = miny; y<miny+spany; y=y+d) {
				GPoint p = GeoModel.createPoint(x+(Math.random()-Math.random())*r,
						y+(Math.random()-Math.random())*r); 
				pset.add(p);
			}
		}
		return pset.toArray();		
	}
	
	
	public static List<GTriangle> generateTriangles() {
		try {
			List<GTriangle> list = new ArrayList<GTriangle>();
			list.add(new GTriangle(new CoordinateM(0,0), new CoordinateM(100,0), new CoordinateM(50,100)));
			return list;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}
	
	
}