package de.hsbo.fbv.bmg.geometry.simple2hd.demos;

import de.hsbo.fbv.bmg.geometry.simple.GPoint;
import de.hsbo.fbv.bmg.geometry.simple.GTriangle;
import de.hsbo.fbv.bmg.geometry.simple.GeoModel;
import de.hsbo.fbv.bmg.geometry.simple2hd.buch.Interporlation2hD;

/**
 * Diese Klasse demonstriert die Interpolation von Hhenwerten in einem Dreieck
 * gem Listing 4.9:<br>
 * Gegeben ist ein Dreieck mit einer horizontalen Grundseite AB (=100 m) und eine 
 * Hhe (Punkt C, im Grundriss) ber AB von 100 m hat. Zugleich weist Punkt C eine
 * z-Differenz von 100 m gegenber der Grundseite auf.<br>
 * Punkt R1 liegt vollstndig im Dreeick, sein z-Wert betrgt 140 m.<br>
 * Punkt R2 liegt auf der Seite AC, sein z-Wert wird daher noch berechnet mit 180 m.<br>
 * Punkte R3 und R4 liegen auerhalb des Dreiecks, ihre Werte betragen Double.NaN.<br>
 * @author A.Zimmermann, HS Bochum, 2011
 *
 */
public class InterpolationDemo {

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		// Dreieck, das nur R1 und R2 berdeckt
		GTriangle triangle = GeoModel.createTriangle(
				GeoModel.createPoint(100,100,100), 	// Punkt A
				GeoModel.createPoint(200,100,100),	// Punkt B
				GeoModel.createPoint(150,200,200));	// Punkt C
		
		// Punkte eines Rechtecks
		GPoint[] points = new GPoint[] {
				GeoModel.createPoint(140, 140),		// Punkt R1
				GeoModel.createPoint(140, 180),		// Punkt R2
				GeoModel.createPoint(230, 140),		// Punkt R3
				GeoModel.createPoint(230, 180)};	// Punkt R4
		
		/*
		// Testpunkte
		GPoint[] points = new GPoint[] {
				GeoModel.createPoint(100, 100),
				GeoModel.createPoint(200, 100),
				GeoModel.createPoint(150, 200),
				GeoModel.createPoint(150, 150)};
		*/
		
		// Interpolation berechnen lassen
		GPoint[] pointsZ;
		try {
			pointsZ = Interporlation2hD.getInterpolatedPointsZ(triangle, points);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return;
		}
		
		// Ausgabe des Ergebnisses
		for(int i=0; i<pointsZ.length; i++)
			System.out.println(pointsZ[i]+ " "+ pointsZ[i].z());
		
	}

}
